"""Assigns HydroID of Junction to JunctionID field in upstream edges.

Each trace starts at a given junction and goes upstream until another junction
in the same layer is found or until the upstream end of the network is found.
Edges returned from the trace get the junction's HydroID value assigned to
their JunctionID field.

Requires:
    Edge and Junction layers participate in the same network.
    Junctions have HydroID (Long Integer) attribute assigned.
    Edges and Junctions have LengthDown (Double) attribute assigned.

LengthDown is used to identify downstream edges that were mistakenly selected
for an upstream trace from a junction.  If the edge LengthDown is less than
the junction LengthDown, then the edge is downstream and should not be assigned
that junction's HydroID.  Using this precomputed attribute is much faster than
computing LengthDown on the fly.

This is not done in an edit session, so you can't undo.

Uncaught errors:
    Null HydroID in junction
    Null LengthDown in junction or edge

"""

import arcpy

from utils import get_workspace, set_progressor, has_field


arcpy.env.overwriteOutput = True

network = arcpy.GetParameter(0)
junction_layer = arcpy.GetParameter(1)
edge_layer = arcpy.GetParameter(2)

# Check that required fields are present
if not has_field(junction_layer, 'HydroID', 'Integer'):
    arcpy.AddError('Junction layer must have a HydroID field of type Long Integer')
    raise arcpy.ExecuteError
if not has_field(junction_layer, 'LengthDown', 'Double'):
    arcpy.AddError('Junction layer must have a LengthDown field of type Double')
    raise arcpy.ExecuteError
if not has_field(edge_layer, 'LengthDown', 'Double'):
    arcpy.AddError('Edge layer must have a LengthDown field of type Double')
    raise arcpy.ExecuteError
edge_layer = arcpy.AddField_management(edge_layer, 'JunctionID', 'LONG')[0]

record_count = int(arcpy.GetCount_management(junction_layer)[0])
if record_count != 0:
    increment = set_progressor(record_count, 'Running traces')
    agg_lines = 'memory/lines'
    arcpy.AddMessage('HydroIDs Processed:')
    i = 0

    with arcpy.da.SearchCursor(junction_layer, ['HydroID', 'LengthDown']) as jcursor:
        for jrow in jcursor:
            hydro_id = jrow[0]  # The value to write to the upstream edges
            length_down = jrow[1]  # We don't want edges further downstream than this junction
            arcpy.AddMessage(str(hydro_id))
            if (i % increment) == 0:
                arcpy.SetProgressorPosition(i)

            start_pt = arcpy.conversion.FeatureClassToFeatureClass(
                in_features=junction_layer,
                out_path='memory',
                out_name='current_junction',
                where_clause='HydroID = ' + str(hydro_id))[0]

            Updated_Trace_Network = arcpy.tn.Trace(
                in_trace_network=network,
                trace_type='UPSTREAM',
                starting_points=start_pt,
                validate_consistency='DO_NOT_VALIDATE_CONSISTENCY',  ## proceed even if we've updated other features
                condition_barriers=[['HydroID', 'DOES_NOT_EQUAL', 'SPECIFIC_VALUE', hydro_id, 'OR']],
                traversability_scope='JUNCTIONS_ONLY',
                result_types=['AGGREGATED_GEOMETRY'],
                trace_name=str(hydro_id),
                aggregated_lines=agg_lines)[0]

            if Updated_Trace_Network:
                lyr = arcpy.SelectLayerByLocation_management(
                    edge_layer, 'SHARE_A_LINE_SEGMENT_WITH', agg_lines)[0]
                edge_count = int(arcpy.GetCount_management(lyr)[0])
                if edge_count > 0:
                    # Sometimes downstream edges are selected. Remove them.
                    arcpy.SelectLayerByAttribute_management(
                        lyr, 'SUBSET_SELECTION', 'LengthDown >= ' + str(length_down))
                    edge_count = int(arcpy.GetCount_management(lyr)[0])
                    if edge_count > 0:
                        arcpy.AddMessage('Upstream edge count: ' + str(edge_count))
                        arcpy.CalculateField_management(lyr, 'JunctionID', hydro_id)
                    else:
                        arcpy.AddMessage('No upstream edges found')
                else:
                    arcpy.AddMessage('No upstream edges found')
            else:
                arcpy.AddMessage('No trace results')
            i += 1

        arcpy.AddMessage('Validating network topology')
        arcpy.SetProgressorLabel('Validating network topology')
        arcpy.ValidateNetworkTopology_tn(network)  # Edits invalidated the network

arcpy.SetParameter(3, edge_layer)
