import arcpy

from utils import get_workspace, set_progressor


arcpy.env.overwriteOutput = True


def length_down(network, start_geom, spatial_ref):
    """Returns length downstream in trace network's spatial reference units.

    start_geom should contain a single point geometry.

    Returns -1 if trace failed or function returned no result.
    """
    pt_fc = arcpy.CreateFeatureclass_management(
        'memory', 'start_pt', 'POINT', spatial_reference=spatial_ref)[0]
    with arcpy.da.InsertCursor(pt_fc, ['SHAPE@']) as cursor:
        cursor.insertRow([start_geom])

    Updated_Trace_Network = arcpy.tn.Trace(
        in_trace_network=network,
        trace_type='DOWNSTREAM',
        starting_points=pt_fc,
        validate_consistency='DO_NOT_VALIDATE_CONSISTENCY',  ## proceed even if we've updated other features
        functions=[['ADD', 'Shape length', '', '', '', '']])[0]
    if Updated_Trace_Network:
        # Find the message like this:
        # Function Result: ADD(Shape length) = 7535.803035.
        lines = arcpy.GetMessages(0).splitlines()
        for line in lines:
            if line.startswith('Function Result: ADD(Shape length)'):
                length = line.split('=')[1].strip()
                if length[-1] == '.':
                    length = length[:-1]
                return length
    return -1


network = arcpy.GetParameter(0)  # trace network
layer = arcpy.GetParameter(1)  # point or line feature layer

record_count = int(arcpy.GetCount_management(layer)[0])
if record_count != 0:
    increment = set_progressor(record_count, 'Running traces')

    # Add LengthDown field if it does not exist
    layer = arcpy.AddField_management(layer, 'LengthDown', 'DOUBLE')[0]
    workspace = get_workspace(layer)

    with arcpy.da.Editor(workspace) as edit:
        feat_class = arcpy.Describe(layer).featureClass
        spatial_ref = feat_class.spatialReference
        fields = ['SHAPE@', 'LengthDown']
        with arcpy.da.UpdateCursor(layer, fields) as cursor:
            for i, row in enumerate(cursor):
                if (i % increment) == 0:
                    arcpy.SetProgressorPosition(i)
                if feat_class.shapeType == 'Polyline':
                    p1 = row[0].firstPoint
                    p2 = row[0].lastPoint
                    length = length_down(network, p1, spatial_ref)
                    length2 = length_down(network, p2, spatial_ref)
                    if length2 < length:
                        length = length2
                else:                    
                    length = length_down(network, row[0].firstPoint, spatial_ref)
                row[1] = length
                cursor.updateRow(row)

    arcpy.AddMessage('Validating network topology')
    arcpy.SetProgressorLabel('Validating network topology')
    arcpy.ValidateNetworkTopology_tn(network)  # Edits invalidated the network

arcpy.SetParameter(2, layer)
