import arcpy

from utils import get_workspace, set_progressor


arcpy.env.overwriteOutput = True


def next_down(network, start_geom, junction_layer, spatial_ref):
    """Returns HydroID of next downstream junction.

    start_geom should contain a single point geometry.

    Returns -1 if trace failed or function returned no result.
    """
    pt_fc = arcpy.CreateFeatureclass_management(
        'memory', 'start_pt', 'POINT', spatial_reference=spatial_ref)[0]
    with arcpy.da.InsertCursor(pt_fc, ['SHAPE@']) as cursor:
        cursor.insertRow([start_geom])

    Updated_Trace_Network = arcpy.tn.Trace(
        in_trace_network=network,
        trace_type='DOWNSTREAM',
        starting_points=pt_fc,
        barriers=junction_layer,
        include_barriers='INCLUDE_BARRIERS',
        validate_consistency='DO_NOT_VALIDATE_CONSISTENCY',  ## proceed even if we've updated other features
        ignore_barriers_at_starting_points='IGNORE_BARRIERS_AT_STARTING_POINTS',
        functions=[['MAX', 'HydroID', '', '', '', '']])[0]
    if Updated_Trace_Network:
        # Find the message like this:
        # Function Result: MAX(HydroID) = 1234.000000.
        lines = arcpy.GetMessages(0).splitlines()
        arcpy.AddMessage('\n'.join(lines))
        for line in lines:
            if line.startswith('Function Result: MAX(HydroID)'):
                id_text = line.split('=')[1].strip()
                if id_text[-1] == '.':
                    id_text = id_text[:-1]
                hydroid = int(float(id_text))
                if hydroid == 0:
                    return -1
                else:
                    return hydroid
    return -1


network = arcpy.GetParameter(0)  # trace network
in_layer = arcpy.GetParameter(1)  # point or line feature layer
junction_layer = arcpy.GetParameter(2)  # must have HydroID field

record_count = int(arcpy.GetCount_management(in_layer)[0])
if record_count != 0:
    arcpy.SetProgressor('step', 'Running traces', 0, record_count, 1)

    # Add NextDownID field if it does not exist
    layer = arcpy.AddField_management(in_layer, 'NextDownID', 'LONG')[0]
    workspace = get_workspace(in_layer)

    with arcpy.da.Editor(workspace) as edit:
        feat_class = arcpy.Describe(in_layer).featureClass
        spatial_ref = feat_class.spatialReference
        fields = ['SHAPE@', 'NextDownID']
        with arcpy.da.UpdateCursor(in_layer, fields) as cursor:
            for i, row in enumerate(cursor):
                arcpy.SetProgressorPosition(i)
                if feat_class.shapeType == 'Polyline':
                    start_pt = row[0].positionAlongLine(0.5, True)  # midpoint
                else:                    
                    start_pt = row[0]
                row[1] = next_down(network, start_pt, junction_layer, spatial_ref)
                cursor.updateRow(row)

    arcpy.AddMessage('Validating network topology')
    arcpy.SetProgressorLabel('Validating network topology')
    arcpy.ValidateNetworkTopology_tn(network)  # Edits invalidated the network

arcpy.SetParameter(2, layer)
